@@ -22,6 +22,7 @@
 #ifndef AZR3_GTK_HPP
 #define AZR3_GTK_HPP
 
+#include <stdint.h>
 #include <vector>
 
 #include <gtkmm.h>
@@ -48,6 +49,8 @@ public:
   void set_program(unsigned char number);
   void clear_programs();
   
+  static Glib::RefPtr<Gdk::Pixmap> pixmap_from_file(const std::string& file, Glib::RefPtr<Gdk::Bitmap>* bitmap = 0);
+  
 protected:
 
   void control_changed(uint32_t index, float new_value);
@@ -77,6 +80,7 @@ protected:
   bool change_mode(bool fx_mode, Gtk::Fixed& fbox);
   void splitpoint_changed();
   void update_program_menu();
+  void update_split_menu();
   Gtk::Menu* create_menu();
   bool popup_menu(GdkEventButton* event, Gtk::Menu* menu);
   void display_scroll(int line, GdkEventScroll* e);
@@ -93,6 +97,7 @@ protected:
   Switch* m_splitswitch;
   Gtk::Adjustment* m_splitpoint_adj;
   Gtk::Menu* m_program_menu;
+  Gtk::Menu* m_split_menu;
   Gdk::Color m_menu_bg;
   Gdk::Color m_menu_fg;
   Gtk::Fixed m_fbox;
