@@ -2,7 +2,7 @@
     
     AZR-3 - An organ synth
     
-    Copyright (C) 2006-2007 Lars Luthman <lars.luthman@gmail.com>
+    Copyright (C) 2006-2010 Lars Luthman <lars.luthman@gmail.com>
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2 as published
@@ -19,17 +19,75 @@
 
 ****************************************************************************/
 
+#include <cstdlib>
 #include <cstring>
 #include <fstream>
 #include <iostream>
+#include <stdexcept>
 
 #include "main.hpp"
-
+#include "optionparser.hpp"
 
 using namespace std;
 
 
 Main::Main(int& argc, char**& argv) : m_ok(false) {
+  
+  /* this is a bit dumb, but the only way I know of to check whether we were
+     started by lashd is to see if lash_extract_args() removes any arguments */
+  int old_argc = argc;  
+  lash_args_t* lash_args = lash_extract_args(&argc, &argv);
+  m_started_by_lashd = (argc != old_argc);
+  
+  // parse all non-LASH arguments
+  OptionParser op;
+  bool help(false);
+  bool version(false);
+  unsigned preset_no(128);
+  string jack_name("AZR-3");
+  try {
+    op.set_env_prefix("AZR3_JACK_")
+      .add_bare("help", "h", help, 
+		"Display this help text and exit.")
+      .add_bare("version", "v", version, 
+		"Display version information and exit.")
+      .add("midi-input", "m", "PORT|CLIENT", m_auto_midi,
+	   "When used, azr3 will try to connect its MIDI input\n"
+	   "port to PORT (if it's a JACK port name) or the\n"
+	   "first MIDI output port in CLIENT (if it's a JACK\n"
+	   "client name).")
+      .add("audio-output", "a", "PORT|CLIENT", m_auto_audio,
+	   "When used, azr3 will try to connect its audio\n"
+	   "output ports to PORT (if it's a JACK port name) or\n"
+	   "the first two audio input ports in CLIENT (if it's\n"
+	   "a JACK client name).")
+      .add("preset", "p", "NUMBER", preset_no,
+	   "Load the preset with the given number instead of\n"
+	   "the first available one.")
+      .add("jack-name", "j", "NAME", jack_name,
+	   "Set the name of the JACK client. The default is\n"
+	   "'AZR-3'. Note that JACK may change this name by\n"
+	   "e.g. adding a number at the end if needed.")
+      .parse_env()
+      .parse(argc, argv);
+  }
+  catch (runtime_error& e) {
+    cerr<<e.what()<<endl;
+    return;
+  }
+  
+  if (help || version) {
+    cout<<argv[0]<<" version "<<PACKAGE_VERSION<<'\n'
+	<<"This JACK version ported by Lars Luthman <lars.luthman@gmail.com>\n"
+	<<"from the VST original by Rumpelrausch Täips\n\n";
+    if (help) {
+      cout<<"Program options:\n\n";
+      op.print_help(cout);
+      op.print_env_help(cout);
+    }
+    cout<<flush;
+    return;
+  }
     
   // initialise controls with default values
   float defaults[] = { 0.00, 0.20, 0.20, 0.00, 0.00, 0.75, 0.50, 0.60, 0.60, 
@@ -58,7 +116,7 @@ Main::Main(int& argc, char**& argv) : m_ok(false) {
   }
     
   // initialise JACK client
-  m_jack_client = jack_client_open("AZR-3", jack_options_t(0), 0);
+  m_jack_client = jack_client_open(jack_name.c_str(), jack_options_t(0), 0);
   m_midi_port = jack_port_register(m_jack_client, "MIDI", 
 				   JACK_DEFAULT_MIDI_TYPE, 
 				   JackPortIsInput, 0);
@@ -96,18 +154,22 @@ Main::Main(int& argc, char**& argv) : m_ok(false) {
   for (uint32_t i = 0; i < 63; ++i)
     m_gui->set_control(i, m_gui_controls[i]);
     
-  // find and use the first preset
-  for (int i = 0; i < 128; ++i) {
-    if (!m_presets[i].empty) {
-      gui_set_preset(i);
-      break;
+  // try to load the desired preset, if it doesn't work use the first one
+  if (preset_no < 128 && !m_presets[preset_no].empty)
+    gui_set_preset(preset_no);
+  else {
+    for (int i = 0; i < 128; ++i) {
+      if (!m_presets[i].empty) {
+	gui_set_preset(i);
+	break;
+      }
     }
   }
     
   // initialise LASH
-  if (!init_lash(argc, argv, jack_get_client_name(m_jack_client)))
+  if (!init_lash(lash_args, jack_get_client_name(m_jack_client)))
     return;
-    
+  
   m_win->set_title("AZR-3");
   m_win->set_resizable(false);
   m_win->add(*m_gui);
@@ -120,6 +182,11 @@ Main::Main(int& argc, char**& argv) : m_ok(false) {
 void Main::run() {
   m_engine->activate();
   jack_activate(m_jack_client);
+
+  // auto-connect JACK ports if desired
+  if (!m_started_by_lashd)
+    auto_connect();
+
   Glib::signal_timeout().
     connect(sigc::bind_return(sigc::mem_fun(*this, &Main::check_changes), 
 			      true), 10);
@@ -335,8 +402,9 @@ bool Main::check_lash_events() {
 }
 
 
-bool Main::init_lash(int& argc, char**& argv, const std::string& jack_name) {
-  m_lash_client = lash_init(lash_extract_args(&argc, &argv), "AZR-3", 
+bool Main::init_lash(lash_args_t* lash_args, const std::string& jack_name) {
+  
+  m_lash_client = lash_init(lash_args, "AZR-3", 
 			    LASH_Config_File, LASH_PROTOCOL(2, 0));
   if (m_lash_client) {
     lash_event_t* event = lash_event_new_with_type(LASH_Client_Name);
@@ -357,6 +425,56 @@ int Main::static_process(jack_nframes_t frames, void* arg) {
 }
 
 
+void Main::auto_connect() {
+
+  const char* env;
+  const char** port_list;
+  const char* name = jack_get_client_name(m_jack_client);
+
+  // MIDI input
+  if (m_auto_midi != "") {
+
+    // if it's a client, connect to the first MIDI output port
+    if (m_auto_midi.find(':') == string::npos &&
+	(port_list = jack_get_ports(m_jack_client, (m_auto_midi + ":*").c_str(),
+				    JACK_DEFAULT_MIDI_TYPE, 
+				    JackPortIsOutput)) && port_list[0]) {
+      jack_connect(m_jack_client, port_list[0], jack_port_name(m_midi_port));
+      free(port_list);
+    }
+
+    // if not, connect to that port
+    else
+      jack_connect(m_jack_client, m_auto_midi.c_str(), 
+		   jack_port_name(m_midi_port));
+  }
+
+  // audio output
+  if (m_auto_audio != "") {
+    
+    // if it's a client, connect individual ports
+    if (m_auto_audio.find(':') == string::npos &&
+	(port_list = jack_get_ports(m_jack_client, 
+				    (m_auto_audio + ":*").c_str(),
+				    JACK_DEFAULT_AUDIO_TYPE, 
+				    JackPortIsInput)) && port_list[0]) {
+      if (port_list[0])
+	jack_connect(m_jack_client, jack_port_name(m_left_port), port_list[0]);
+      if (port_list[1])
+	jack_connect(m_jack_client, jack_port_name(m_right_port), port_list[1]);
+      free(port_list);
+    }
+
+    // if not, connect all our ports to that single port
+    else {
+      jack_connect(m_jack_client, 
+		   jack_port_name(m_left_port), m_auto_audio.c_str());
+      jack_connect(m_jack_client, 
+		   jack_port_name(m_right_port), m_auto_audio.c_str());
+    }
+  }
+}
+
 
 
 int main(int argc, char** argv) {
